/*
Copyright (C) 2003-2007 Anders Ervik <calvin@countzero.no>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once
#include "filereader.h"
#include "master.h"

namespace zzub {

const unsigned int MAGIC_Buzz=0x7a7a7542;
const unsigned int MAGIC_MACH=0x4843414d;
const unsigned int MAGIC_CONN=0x4E4E4F43;
const unsigned int MAGIC_PATT=0x54544150;
const unsigned int MAGIC_SEQU=0x55514553;
const unsigned int MAGIC_WAVT=0x54564157;
const unsigned int MAGIC_CWAV=0x56415743;
const unsigned int MAGIC_WAVE=0x45564157;
const unsigned int MAGIC_BLAH=0x48414c42;
const unsigned int MAGIC_PARA=0x41524150;
const unsigned int MAGIC_MIDI=0x4944494D;


struct Section {
	unsigned int magic;
	unsigned int offset;
	unsigned int size;
};


class MachineParameterInformation {
public:
	std::vector<MachineValidation> machines;
};

/*
struct metaplugin_alias {
	metaplugin_alias(std::string alias, std::string uri) { this->alias=alias; this->uri=uri; }

	std::string alias;
	std::string uri;
};

struct BuzzCompatibility {
    std::vector<metaplugin_alias> mappings;

    void registerBuzzUri(std::string pluginName, std::string uri);
    std::string getBuzzUri(std::string pluginName);
    std::string getBuzzName(std::string uri);
};*/

class BuzzReader : public FileReader, public zzub::event_handler {

	unsigned int sectionCount;
	Section* sections;
	zzub::player* player;
	Section* getSection(unsigned int magic);

    bool loadMachines();
	bool loadPatterns();
	bool loadConnections();
	bool loadSequences();
	bool loadWaveTable();
	bool loadWaves();
	bool loadPara();
	bool loadMidi();
	bool loadInfoText();

	void clear();
	MachineParameterInformation machineParameters;

	bool testMachineCompatibility(zzub::metaplugin* machine);
	MachineValidation* findMachinePara(std::string name, std::string fullName);

	//bool ZZUB_STDCALL validateMachine(void* param);
    bool invoke(zzub_event_data_t& data);

	bool init(zzub::player* pl, const char* fn);

public:
	std::vector<zzub::metaplugin*> machines;
	std::string lastError;
	std::string lastWarning;

	BuzzReader();
	~BuzzReader();
	void loadTrack(zzub::metaplugin* machine, patterntrack* track);
	bool open(zzub::player* pl, const char* fn);
	//zzub::metaplugin* createDummyMachine(int type, std::string fullName, std::string machineName, int attributeCount);
};


class reader_instream : public zzub::instream {
	StreamReader& reader;
public:
	reader_instream(StreamReader& br);
	virtual void read(void *pbuf, int numbytes);
	long position();
	void seek(long pos, int where);
};

}
