/*
Copyright (C) 2003-2007 Anders Ervik <calvin@countzero.no>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "common.h"
#include "bmxwriter.h"
#include "compress.h"

using namespace std;

namespace zzub {

BuzzWriter::BuzzWriter(void) {
}

BuzzWriter::~BuzzWriter(void) {
}

void saveTrack(FileWriter& f, patterntrack* track) {
	for (size_t j=0; j<(size_t)track->getRows(); j++) {
		for (size_t k=0; k<track->getParams(); k++) {
			const zzub::parameter* param=track->getParam(k);
			unsigned short value=track->getValue(j, k);
			f.writeBytes(&value, param->get_bytesize());
		}
	}
}


bool BuzzWriter::validateClassesForSave() {
    bool returnValue=true;
    player->loadError="";
    for (size_t i=0; i<player->getMachines(); i++) {
        metaplugin* plugin=player->getMachine(i);
        if (plugin->getType()==zzub_plugin_type_master) continue;

        string pluginName=player->getBuzzName(plugin->loader->getUri());
        if (pluginName=="")
            pluginName=plugin->loader->getUri();
        if (!pluginName.size()) {
            player->loadError+=plugin->getLoaderName() + " is not registered in buzzcompat.txt\n";
            returnValue=false;
        }
    }
    return returnValue;
}


bool BuzzWriter::save(zzub::player* pl, const char* fn) {
	player=pl;

	const int sectionCount=31;	// MACH, PATT, CONN, SEQU, WAVT, WAVE

	numberOfWaves=0;

    if (!validateClassesForSave()) return false;

    presortPatterns();

	machines.clear();

	if (!create(fn)) return false;
	write(MAGIC_Buzz);
	write(sectionCount);

	size_t pos=position();

	// leave space for section declarations
	int i;	// for vc6
	for (i=0; i<sectionCount*sizeof(Section); i++) {
		write(0);
	}

	Section sections[sectionCount];
	memset(sections, 0, sizeof(Section)*sectionCount);

	sections[0].magic=MAGIC_MACH;
	sections[0].offset=position();
	saveMachines();
	sections[0].size=position()-sections[0].offset;

	sections[1].magic=MAGIC_PATT;
	sections[1].offset=position();
	savePatterns();
	sections[1].size=position()-sections[1].offset;

	sections[2].magic=MAGIC_CONN;
	sections[2].offset=position();
	saveConnections();
	sections[2].size=position()-sections[2].offset;

	sections[3].magic=MAGIC_SEQU;
	sections[3].offset=position();
	saveSequences();
	sections[3].size=position()-sections[3].offset;

	sections[4].magic=MAGIC_WAVT;
	sections[4].offset=position();
	saveWaveTable();
	sections[4].size=position()-sections[4].offset;

	sections[5].magic=MAGIC_CWAV;
	sections[5].offset=position();
	saveWaves();
	sections[5].size=position()-sections[5].offset;

	sections[6].magic=MAGIC_BLAH;
	sections[6].offset=position();
	saveComment(player->infoText);
	sections[6].size=position()-sections[6].offset;

	sections[7].magic=MAGIC_PARA;
	sections[7].offset=position();
	savePara();
	sections[7].size=position()-sections[7].offset;

	sections[8].magic=MAGIC_MIDI;
	sections[8].offset=position();
	saveMidi();
	sections[8].size=position()-sections[8].offset;

	seek((long)pos);
	for (i=0; i<sectionCount; i++) {
		write(sections[i].magic);
		write(sections[i].offset);
		write(sections[i].size);
	}

	close();
	return true;
}

bool BuzzWriter::saveMidi() {

    for (size_t i=0; i<player->getMachines(); i++) {
        metaplugin* m=player->getMachine(i);
        for (size_t j=0; j<player->getMidiMappings(); j++) {
            midimapping* mm=player->getMidiMapping(j);
            if (mm->machine==m) {
                writeAsciiZ(m->getName().c_str());
                write((char)mm->group);
                write((char)mm->track);
                write((char)mm->column);
                write((char)mm->channel);
                write((char)mm->controller);
            }
        }
    }
    write((char)0);

    return true;
}


bool BuzzWriter::savePara() {
	write((unsigned int)player->getMachines());
	for (size_t i=0; i<player->getMachines(); i++) {
		zzub::metaplugin* m=player->getMachine(i);
		writeAsciiZ(m->getName().c_str());
        string pluginName=player->getBuzzName(m->loader->getUri());
        if (pluginName=="")
            pluginName=m->loader->getUri();
		writeAsciiZ(pluginName.c_str());
        size_t numGlobals = m->machineInfo->global_parameters.size();
		write(numGlobals);
		size_t numTrackParams = m->machineInfo->track_parameters.size();
		write(numTrackParams);

		for (size_t j=0; j<numGlobals; j++) {
			const zzub::parameter* cmp = m->machineInfo->global_parameters[j];
			write((char)cmp->type);
			writeAsciiZ(cmp->name);
			write(cmp->value_min);
			write(cmp->value_max);
			write(cmp->value_none);
			write(cmp->flags);
			write(cmp->value_default);
		}

		for (size_t j=0; j<numTrackParams; j++) {
			const zzub::parameter* cmp = m->machineInfo->track_parameters[j];
			write((char)cmp->type);
			writeAsciiZ(cmp->name);
			write(cmp->value_min);
			write(cmp->value_max);
			write(cmp->value_none);
			write(cmp->flags);
			write(cmp->value_default);
		}

	}
	return true;
}

bool BuzzWriter::saveMachine(zzub::metaplugin* machine) {
	zzub::metaplugin* m=machine;

	writeAsciiZ(m->getName().c_str());
	write((char)m->getType());

	// nr vi var buzzlib var det writeAsciiZ(m->getFullName().c_str());
	// n er det zzublib, og da blir det slik:
	if (m->getType()!=zzub::plugin_type_master) {
		std::string machineName=player->getBuzzName(m->loader->getUri());//this->player->getAliasForMachine(m);
        if (!machineName.length())
            machineName=m->loader->getUri();
        writeAsciiZ(machineName.c_str());
	}
	write(m->x);
	write(m->y);

	int n=0;
	int p1=position();
	write(n);	// save space for machine data size

	m->save(*this);

	int p2=position();
	seek(p1);
	write((unsigned int)(p2-(p1+4)));	// update machine data size
	seek(p2);

	write((unsigned short)m->getAttributes());
	size_t i;
	for (i=0; i<m->getAttributes(); i++) {
		const zzub::attribute& attr=m->getAttribute(i);
		writeAsciiZ(attr.name);
		write(m->getAttributeValue(i));
	}

	// save global default parameters
	saveTrack(*this, m->getStateTrackCopy(1, 0));

	// save track default parameters
	write((unsigned short)m->getTracks());

	for (i=0; i<m->getTracks(); i++) {
		saveTrack(*this, m->getStateTrackCopy(2, (int)i));
	}
	return true;
}


bool BuzzWriter::saveMachines() {
	write((unsigned short)player->getMachines());

	bool first=true;

	// save master as the first machine
	for (size_t i=0; i<player->getMachines(); i++) {
		zzub::metaplugin* m=player->getMachine(i);//i->second;
		if (m==player->getMaster()) {
			saveMachine(m);
			machines.push_back(m);
			break;
		}
	}

	// save the rest of the machines
	for (size_t i=0; i<player->getMachines(); i++) {
		zzub::metaplugin* m=player->getMachine(i);
		if (m!=player->getMaster()) {
			saveMachine(m);
			machines.push_back(m);
		}
	}
	return true;
}

int pattern_sorter(pattern* p1, pattern* p2) {
    return p1->getName() < p2->getName();
}

// buzz expects sorted patterns
void BuzzWriter::presortPatterns() {
	for (size_t i=0; i<player->getMachines(); i++) {
		zzub::metaplugin* m=player->getMachine(i);
        
        std::vector<pattern*> patterns(m->getPatterns());

		for (size_t j=0; j<m->getPatterns(); j++) {
            patterns[j] = m->getPattern(j);
        }
        std::sort(patterns.begin(), patterns.end(), pattern_sorter);

        m->patterns = patterns;
    }
}

bool BuzzWriter::savePatterns() {
	for (size_t i=0; i<machines.size(); i++) {
		zzub::metaplugin* m=machines[i];

		write((unsigned short)m->getPatterns());
		write((unsigned short)m->getTracks());

		for (size_t j=0; j<m->getPatterns(); j++) {
			pattern* p=m->getPattern(j);
			writeAsciiZ(p->getName().c_str());
			write((unsigned short)p->getRows());

			// lagre connection patterns 
			for (size_t k=0; k<m->getConnections(); k++) {
				// lese amps pr connection nr det er effekter
				zzub::metaplugin* cmac=m->getConnection(k)->plugin_in;
				write((unsigned short)getMachineIndex(cmac));
				saveTrack(*this, p->getPatternTrack(0, k));
			}
			saveTrack(*this, p->getPatternTrack(1, 0));

			for (int l=0; l<(int)m->getTracks(); l++) {
				saveTrack(*this, p->getPatternTrack(2, l));
			}
		}
	}
	return true;
}



bool BuzzWriter::saveConnections() {
	int p1=position();
	unsigned short n=0;
	write(n);
	int conns=0;
	for (size_t i=0; i<machines.size(); i++) {
		zzub::metaplugin* m=machines[i];
		for (size_t j=0; j<m->getConnections(); j++) {
			connection* cx=m->getConnection(j);
			unsigned short i1=(unsigned short)getMachineIndex(cx->plugin_in);
			unsigned short i2=(unsigned short)getMachineIndex(cx->plugin_out);
			write((unsigned short)i1);
			write((unsigned short)i2);
			write((unsigned short)cx->amp);
			write((unsigned short)cx->pan);

			conns++;
		}
	}
	int p2=position();
	seek(p1);
	write((unsigned short)conns);
	seek(p2);
	return true;
}

bool BuzzWriter::saveSequences() {
	write((unsigned int)player->getSongEndLoop());
	write((unsigned int)player->getSongBeginLoop());
	write((unsigned int)player->getSongEndLoop());
	write((unsigned short)player->getSequenceTracks());
	for (int i=0; i<(int)player->getSequenceTracks(); i++) {
		sequence* tr=player->getSequenceTrack(i);
		write((unsigned short)getMachineIndex(tr->getMachine()));
		tr->serialize(*this);
	}
	return true;
}

bool BuzzWriter::saveWaveTable() {
	numberOfWaves=0;

	//find how many waves are really used
	size_t i;	// for vc6
	for (i=0; i<player->getWaves(); i++) {
		wave_info_ex* entry=player->getWave(i);
		if (entry->getLevels()>0)
			numberOfWaves++;
	}

	write(numberOfWaves);

	for (i=0; i<player->getWaves(); i++) {
		wave_info_ex* entry=player->getWave(i);;
		if (entry->getLevels()==0) continue;
		write((unsigned short)i);
		writeAsciiZ(entry->fileName.c_str());
		writeAsciiZ(entry->name.c_str());
		write(entry->volume);

		//entry.Flags&=WF_STEREO;	// ?? only support stereo

		write((unsigned char)entry->flags);

		if ((entry->flags&zzub::wave_flag_envelope)!=0) {
			write((unsigned short)entry->envelopes.size());

			for (size_t j=0; j<entry->envelopes.size(); j++) {
				envelope_entry& env=entry->envelopes[j];
				write(env.attack);
				write(env.decay);
				write(env.sustain);
				write(env.release);
				write(env.subDivide);
				write(env.flags);

				unsigned short numPoints=env.points.size();
				if (env.disabled)
					numPoints|=0x8000;
				write(numPoints);
				for (size_t k=0; k<env.points.size(); k++) {
					envelope_point& pt=env.points[k];
					write(pt.x);
					write(pt.y);
					write(pt.flags);
				}
			}
		}

		write((unsigned char)entry->getLevels());
		for (size_t j=0; j<entry->getLevels(); j++) {
			zzub::wave_level* level=entry->getLevel(j);
			write(level->sample_count);
			write(level->loop_start);
			write(level->loop_end);
			write(level->samples_per_second);
			write((unsigned char)level->root_note);
		}
	}

	return true;
}

bool BuzzWriter::saveWaves() {
	write(numberOfWaves);

	for (size_t i=0; i<player->getWaves(); i++) {
		wave_info_ex* entry=player->getWave(i);
		if (entry->getLevels()==0) continue;
		write((unsigned short)i);

		unsigned int waveSize=0;
		char format=1;
		write(format);
		
		WAVEUNPACK wup;
		long pos=position();
		if (format==0) {
			write(waveSize);
		} else
		if (format==1) {
			InitWavePack(&wup, f);
		}

		int numChannels=entry->isStereo()?2:1;
		for (size_t j=0; j<entry->getLevels(); j++) {
			zzub::wave_level& level=*entry->getLevel(j);

			// save wave data
			if (format==1) {

				CompressWave(&wup, (LPWORD)level.samples, level.sample_count, entry->isStereo()?TRUE:FALSE);
			} else 
			if (format==0) {
				writeBytes(level.samples, level.sample_count*numChannels*sizeof(short));
			}
		}

		if (format==1) {
			FlushPackedBuffer(&wup,TRUE);
		}

		if (format==0) {
			long pos2=position();
			seek(pos);
			write(waveSize);
			seek(pos2);
		}
	}
	return true;
}

bool BuzzWriter::saveComment(std::string text) {
	write((unsigned int)text.length());
	writeBytes((void*)text.c_str(), text.length());
	return true;
}

size_t BuzzWriter::getMachineIndex(zzub::metaplugin* m) {
	using namespace std;
	for (size_t i=0; i<machines.size(); i++) {
		if (machines[i]==m) return i;
	}
	return -1;
}

//void CMachineDataOutput::Write(void *pbuf, int const numbytes) { }

CBuzzOutput::CBuzzOutput(StreamWriter& br):writer(br) {
}

void CBuzzOutput::write(void *pbuf, int numbytes) {
	writer.writeBytes(pbuf, numbytes);
}

}
