/*
Copyright (C) 2003-2007 Anders Ervik <calvin@countzero.no>
Copyright (C) 2006-2007 Leonard Ritter

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#include <string>

class RtAudio;


namespace zzub {

struct audioworker
{
    virtual void workStereo(float** buffer, int num) {}
	virtual ~audioworker() {}
};

struct audiodevice {
    int api_id;
    int device_id;
    std::string name;
    int in_channels;
    int out_channels;
    std::vector<int> rates;
};

struct audiodriver
{
	enum {
		// increase this if you get problems
		MAX_FRAMESIZE = 16384,
	};
	
	audioworker *worker;
	RtAudio *audio;
	bool started;
    bool recording;
	float workbufferL[MAX_FRAMESIZE];
	float workbufferR[MAX_FRAMESIZE];
	unsigned int desired_framecount;
	unsigned int desired_framerate;

    int defaultDevice;
    std::vector<audiodevice> devices;

	audiodriver();

    int getApiDevices(int apiId);

	virtual ~audiodriver();	
	virtual void initialize(audioworker *worker);	
	virtual void reset();
	virtual bool enable(bool e);	
	virtual int getWritePos();	
	virtual int getPlayPos();	
	virtual void configure();
	
	virtual int getDeviceCount();
	virtual std::string getDeviceName(int index);
	virtual bool createDevice(int index, int inIndex=-1);
	virtual void destroyDevice(); 	
	
	//~ virtual PaHostApiTypeId getHostApiTypeIdForName(const char *name);
	//~ virtual int getBestDeviceForHostApi(PaHostApiTypeId hostapiid);
	virtual int getBestDevice();

    virtual int getDeviceByName(const char* name);
    virtual bool isSampleRateSupported(int index, unsigned int rate);
    virtual bool isOutputDevice(int index);
    virtual bool isInputDevice(int index);
	
	void setSampleRate(unsigned int samplerate);
	unsigned int getSampleRate();
	void setBufferSize(unsigned int buffersize);
	unsigned int getBufferSize();

    double getCpuLoad();
    bool isRecording();
};

}
