/*
Copyright (C) 2003-2007 Anders Ervik <calvin@countzero.no>
Copyright (C) 2006-2007 Leonard Ritter

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "common.h"
#include "tools.h"
#include "driver.h"
#include <assert.h>

using namespace std;

namespace zzub {


struct input_plugin : plugin {
	player *_player;
	
	input_plugin(player *p) {
		assert(p);
		this->_player = p;
	}
	
	virtual void destroy() { delete this; }
	virtual void init(zzub::archive *arc) {}
	virtual void process_events() {}
	virtual bool process_stereo(float **pin, float **pout, int numsamples, int mode) {
		if( (mode&zzub::process_mode_write)==0 )
			return false;
		if (_player->inputBuffer[0] == 0 || _player->inputBuffer[1] == 0) 
			return false;
		memcpy(pout[0], _player->inputBuffer[0], sizeof(float) * numsamples);
		memcpy(pout[1], _player->inputBuffer[1], sizeof(float) * numsamples);
		return true;
	}
	virtual void stop() {}
	virtual void save(zzub::archive *arc) {}
	virtual void attributes_changed() {}
	virtual void command(int index) {}
	virtual void set_track_count(int count) {}
	virtual void mute_track(int index) {}
	virtual bool is_track_muted(int index) const { return false; }
	virtual void midi_note(int channel, int value, int velocity)  {}
	virtual void event(unsigned int data)  {}
	virtual const char * describe_value(int param, int value) { return 0; }
	virtual const zzub::envelope_info ** get_envelope_infos() { return 0; }
	virtual bool play_wave(int wave, int note, float volume) { return false; }
	virtual void stop_wave() {}
	virtual int get_wave_envelope_play_position(int env) { return -1; }

	// these have been in zzub::plugin2 before
	virtual const char* describe_param(int param) { return 0; }
	virtual bool set_instrument(const char *name) { return false; }
	virtual void get_sub_menu(int index, zzub::outstream *os) {}
	virtual void add_input(const char *name) {}
	virtual void delete_input(const char *name) {}
	virtual void rename_input(const char *oldname, const char *newname) {}
	virtual void input(float **samples, int size, float amp) {}
	virtual void midi_control_change(int ctrl, int channel, int value) {}
	virtual bool handle_input(int index, int amp, int pan) { return false; }
};

zzub::plugin* input_plugin_info::create_plugin() const { 
	return new input_plugin(_player);
}

void input_plugincollection::setPlayer(player *p) {
	_info._player = p;
}

void input_plugincollection::initialize(zzub::pluginfactory *factory) {
	factory->register_info(&_info);
}

input_plugincollection the_input_plugincollection;

plugincollection *get_input_plugincollection(audiodriver *ad) {
	return &the_input_plugincollection;
}

} // namespace zzub
