/*
Copyright (C) 2003-2007 Anders Ervik <calvin@countzero.no>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

namespace zzub {

struct envelope_point {
	unsigned short x, y;
	unsigned char flags;		// flags: bit 0 = sustain
};

struct envelope_entry {
	unsigned short attack, decay, sustain, release;
	char subDivide, flags;	// ADSR Subdivide & Flags
	bool disabled;
	std::vector<envelope_point> points;

	envelope_entry();
	void clear();
};

struct wave_info_ex;

struct wave_level_ex : wave_level {
	wave_info_ex* wave;
	int level;

	wave_level_ex();
};

struct wave_info_ex : wave_info {
	std::vector<wave_level_ex> levels;
	std::string fileName;
	std::string name;
	std::vector<envelope_entry> envelopes;

	wave_info_ex();
	~wave_info_ex();
	void clear();
	bool isStereo() const;
	bool isExtended() const;
	void setStereo(bool state);
	void setExtendedMode();	// can only set to extended mode, no reversion supported
	size_t getLevels() const ;
	wave_level_ex* getLevel(size_t level) const;
	bool saveLevel(size_t level, const char* fn);
	bool allocateLevel(size_t level, size_t samples, int waveFormat);	// waveFormat, WFB_* ... 2=32 signed, 3=24 signed, ?=32 float
	bool reallocateLevel(size_t level, size_t samples);	// waveFormat, WFB_* ... 2=32 signed, 3=24 signed, ?=32 float
	void* getSampleData(size_t level) const;
	size_t getBitsPerSample(size_t level) const;
	size_t getNumSamples(size_t level) const;
	int getRootNote(size_t level) const;
	size_t getLoopStart(size_t level) const;
	size_t getLoopEnd(size_t level) const;
	size_t getSamplesPerSec(size_t level) const;
	void setRootNote(size_t level, size_t value);
	void setLoopStart(size_t level, size_t value);
	void setLoopEnd(size_t level, size_t value);
	void setSamplesPerSec(size_t level, size_t value);
	int getWaveFormat(size_t level) const;
	bool createWaveRange(size_t level, size_t fromSample, size_t numSamples, void** sampleData);
	bool silenceWaveRange(size_t level, size_t fromSample, size_t numSamples);
	bool removeWaveRange(size_t level, size_t fromSample, size_t numSamples);
	bool insertWaveAt(size_t level, size_t atSample, void* sampleData, size_t channels, int waveFormat, size_t numSamples);
	size_t getLevelIndex(wave_level_ex* level);
	void setLooping(bool state);
	void setBidir(bool state);
	bool isLooping();
	bool isBidir();
};

struct wave_table {
	std::vector<wave_info_ex> waves;
	wave_info_ex monitorwave; // for prelistening

	wave_table(void);
	~wave_table(void);
	void clear();
};

};
