/*****************************************************************************

        Biquad.h
        Copyright (c) 2002-2006 Laurent de Soras

--- Legal stuff ---

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*Tab=3***********************************************************************/



#if ! defined (dsp_Biquad_HEADER_INCLUDED)
#define	dsp_Biquad_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250) // "Inherits via dominance."
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



namespace dsp
{



// Can be inherited but is not polymorph.
class Biquad
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

						Biquad ();
						~Biquad () {}

	inline void		set_z_eq (const float b [3], const float a [3]);
	inline void		copy_filter (const Biquad &other);

	inline float	process_sample (float x);

	void				process_block (float *dest_ptr, const float *src_ptr, long nbr_spl);
	void				process_block (float *spl_ptr, long nbr_spl);

	void				clear_buffers ();



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:

	float				_z_eq_b [3];	// Direct coefficients, order z^(-n)
	float				_z_eq_a [3];	// Recursive coefficients, order z^(-n)



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	float				_mem_x [2];		// Input memory, order z^(-n)
	float				_mem_y [2];		// Output memory, order z^(-n)
	int				_mem_pos;		// 0 or 1



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						Biquad (const Biquad &other);
	Biquad &			operator = (const Biquad &other);
	bool				operator == (const Biquad &other);
	bool				operator != (const Biquad &other);

};	// class Biquad



}	// namespace dsp



#include	"Biquad.hpp"



#endif	// dsp_Biquad_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
